/*
 * Copyright (C) 2016 Nishant Srivastava
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package main.java.github.nisrulz.qreader;

import android.Manifest;
import android.content.Context;
import android.os.Bundle;
import android.content.pm.PackageManager;
import android.support.v4.app.ActivityCompat;
import android.util.Log;
import android.util.SparseArray;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

import main.java.github.nisrulz.qreader.CameraSource;
import com.google.android.gms.vision.Detector;
import com.google.android.gms.vision.barcode.Barcode;
import com.google.android.gms.vision.barcode.BarcodeDetector;


import anywheresoftware.b4a.BA;

import java.io.IOException;

import android.hardware.Camera;
import android.hardware.Camera.Parameters;

import android.app.ActivityManager;
import android.content.ComponentName;

import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import android.support.annotation.StringDef;



import java.util.List;

public class QREader {

    private static final String TAG = "QREader";
    private static BarcodeDetector barcodeDetector;
    private static CameraSource cameraSource;

	
    @StringDef({
        Camera.Parameters.FLASH_MODE_ON,
        Camera.Parameters.FLASH_MODE_OFF,
        Camera.Parameters.FLASH_MODE_AUTO,
        Camera.Parameters.FLASH_MODE_RED_EYE,
        Camera.Parameters.FLASH_MODE_TORCH
    })
    @Retention(RetentionPolicy.SOURCE)
    private @interface FlashMode {}
	

    public static void start(final Context context, final SurfaceView surfaceView, final QRDataListener QRDataListener) {
        barcodeDetector =
                new BarcodeDetector.Builder(context)
                        .setBarcodeFormats(Barcode.ALL_FORMATS)
                        .build();

        cameraSource = new CameraSource
                .Builder(context, barcodeDetector)
                .setRequestedPreviewSize(800, 800)
				.setFocusMode(Camera.Parameters.FOCUS_MODE_AUTO)
				.setFacing(CameraToUse)
				.setRequestedFps(15.0f)
                .build();
	
        //cameraSource.setFlashMode(Camera.Parameters.FLASH_MODE_TORCH);
		
				//.setRequestedPreviewSize(1600, 1024)
				//.setRequestedFps(15.0f);
				
		//BA.Log("cameraSource 1 = " + cameraSource);

        surfaceView.getHolder()
                .addCallback(new SurfaceHolder.Callback() {
                    @Override
                    public void surfaceCreated(SurfaceHolder holder) {
                        startCameraView(context, cameraSource, surfaceView);					
						
                    }

                    @Override
                    public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {

                    }

                    @Override
                    public void surfaceDestroyed(SurfaceHolder holder) {
                        //startCameraView(context, cameraSource, surfaceView);
                    }
                });

        barcodeDetector.setProcessor(new Detector.Processor<Barcode>() {
            @Override
            public void release() {

            }

            @Override
            public void receiveDetections(Detector.Detections<Barcode> detections) {
                final SparseArray<Barcode> barcodes = detections.getDetectedItems();
                
                if (barcodes.size() != 0) {
                    QRDataListener.onDetected(barcodes.valueAt(0).displayValue);
                }
            }
        });

    }

    private static void startCameraView(Context context, CameraSource cameraSource, SurfaceView
            surfaceView) {
        try {
            if (ActivityCompat.checkSelfPermission(context, Manifest.permission.CAMERA)
                    == PackageManager.PERMISSION_GRANTED) {
                cameraSource.start(surfaceView.getHolder());
            } else {
                Log.e(TAG, "Permission not granted!");
            }
        } catch (IOException ie) {
            Log.e(TAG, ie.getMessage());
            ie.printStackTrace();
        }

    }
	
	
    public static void stop(final Context context, final SurfaceView surfaceView) {

        stopCameraView(context, cameraSource, surfaceView);

    }	
	
    private static void stopCameraView(Context context, CameraSource cameraSource, SurfaceView surfaceView) {

	    //BA.Log("camera source = " + cameraSource);
	    cameraSource.stop();
        cameraSource.release();
		cameraSource = null;
		
/*          ActivityManager manager = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
         List<ActivityManager.RunningAppProcessInfo> tasks = manager.getRunningAppProcesses();
         BA.Log("current_app " + tasks.get(0).processName);	

        ActivityManager am =(ActivityManager)context.getSystemService(Context.ACTIVITY_SERVICE);

        // get the info from the currently running task
        List< ActivityManager.RunningTaskInfo > taskInfo = am.getRunningTasks(1);

        BA.Log("topActivity CURRENT Activity :: "
                    + taskInfo.get(0).topActivity.getClassName());

        ComponentName componentInfo = taskInfo.get(0).topActivity;
        //--- To get currently active activity in foreground. We can use getShortClassName() & getClassName()
         String classname =componentInfo.getClassName();
        //---get package name of currently running application in foreground. We can use getPackageName()
        String packagename =componentInfo.getPackageName();
 */		 
		
    }	
	
	private static int CameraToUse = 0;
	public void setCameraToUse(int CameraToUse) {
		this.CameraToUse = CameraToUse;
	}
	
	private static boolean UseFlash = false;
	public void setUseFlash(boolean UseFlash) {
		this.UseFlash = UseFlash;
	}
	
	
	
}

